import { z } from "zod";
import { ApiContextStorage } from "../store.js";
import {
  ActionError,
  ActionInputError
} from "./shared.js";
export * from "./shared.js";
import { z as z2 } from "zod";
function defineAction({
  accept = "json",
  input: inputSchema,
  handler
}) {
  const serverHandler = async (unparsedInput) => {
    const context = ApiContextStorage.getStore();
    if (!inputSchema)
      return await handler(unparsedInput, context);
    if (unparsedInput instanceof FormData) {
      if (accept === "json") {
        throw new ActionError({
          code: "UNSUPPORTED_MEDIA_TYPE",
          message: "This action only accepts JSON input. To accept form data, add `accept: 'form'` to your action configuration."
        });
      }
      unparsedInput = upgradeFormData(unparsedInput, inputSchema);
    }
    const parsed = await inputSchema.safeParseAsync(unparsedInput);
    if (!parsed.success) {
      throw new ActionInputError(parsed.error.issues);
    }
    return await handler(parsed.data, context);
  };
  serverHandler.safe = async () => {
    throw new ActionError({
      code: "INTERNAL_SERVER_ERROR",
      message: "safe() unexpectedly called on the server. To retrieve action data from Astro frontmatter, use the `Astro.getActionResult()` function."
    });
  };
  return serverHandler;
}
function upgradeFormData(formData, schema) {
  const obj = {};
  for (const [key, baseValidator] of Object.entries(schema.shape)) {
    let validator = baseValidator;
    if (baseValidator instanceof z.ZodOptional || baseValidator instanceof z.ZodNullable) {
      validator = baseValidator._def.innerType;
    }
    if (validator instanceof z.ZodBoolean) {
      obj[key] = formData.has(key);
    } else if (validator instanceof z.ZodArray) {
      const entries = Array.from(formData.getAll(key));
      const elementValidator = validator._def.type;
      if (elementValidator instanceof z.ZodNumber) {
        obj[key] = entries.map(Number);
      } else if (elementValidator instanceof z.ZodBoolean) {
        obj[key] = entries.map(Boolean);
      } else {
        obj[key] = entries;
      }
    } else if (validator instanceof z.ZodNumber) {
      obj[key] = Number(formData.get(key));
    } else {
      obj[key] = formData.get(key);
    }
  }
  return obj;
}
export {
  defineAction,
  z2 as z
};
