class ActionError extends Error {
  type = "AstroActionError";
  code = "INTERNAL_SERVER_ERROR";
  status = 500;
  constructor(params) {
    super(params.message);
    this.code = params.code;
    this.status = ActionError.codeToStatus(params.code);
  }
  static codeToStatus(code) {
    switch (code) {
      case "BAD_REQUEST":
        return 400;
      case "UNAUTHORIZED":
        return 401;
      case "FORBIDDEN":
        return 403;
      case "NOT_FOUND":
        return 404;
      case "UNSUPPORTED_MEDIA_TYPE":
        return 415;
      case "INTERNAL_SERVER_ERROR":
        return 500;
    }
  }
  static statusToCode(status) {
    switch (status) {
      case 400:
        return "BAD_REQUEST";
      case 401:
        return "UNAUTHORIZED";
      case 403:
        return "FORBIDDEN";
      case 404:
        return "NOT_FOUND";
      case 415:
        return "UNSUPPORTED_MEDIA_TYPE";
      case 500:
        return "INTERNAL_SERVER_ERROR";
      default:
        return "INTERNAL_SERVER_ERROR";
    }
  }
  static async fromResponse(res) {
    if (res.status === 400 && res.headers.get("Content-Type")?.startsWith("application/json")) {
      const body = await res.json();
      if (typeof body === "object" && body?.type === "AstroActionInputError" && Array.isArray(body.issues)) {
        return new ActionInputError(body.issues);
      }
    }
    return new ActionError({
      message: res.statusText,
      code: this.statusToCode(res.status)
    });
  }
}
function isInputError(error) {
  return error instanceof ActionInputError;
}
class ActionInputError extends ActionError {
  type = "AstroActionInputError";
  // We don't expose all ZodError properties.
  // Not all properties will serialize from server to client,
  // and we don't want to import the full ZodError object into the client.
  issues;
  fields;
  constructor(issues) {
    super({ message: "Failed to validate", code: "BAD_REQUEST" });
    this.issues = issues;
    this.fields = {};
    for (const issue of issues) {
      if (issue.path.length > 0) {
        const key = issue.path[0].toString();
        this.fields[key] ??= [];
        this.fields[key]?.push(issue.message);
      }
    }
  }
}
async function callSafely(action, input) {
  try {
    const data = await action(input);
    return { data, error: void 0 };
  } catch (e) {
    if (e instanceof ActionError) {
      return { data: void 0, error: e };
    }
    return {
      data: void 0,
      error: new ActionError({
        message: e instanceof Error ? e.message : "Unknown error",
        code: "INTERNAL_SERVER_ERROR"
      })
    };
  }
}
function getNameProps(action) {
  return {
    type: "hidden",
    name: "_astroAction",
    value: action.toString()
  };
}
export {
  ActionError,
  ActionInputError,
  callSafely,
  getNameProps,
  isInputError
};
